#include "bme280.h"
#include "hal_data.h"
#include "i2c.h"

uint16_t dig_T1;
int16_t dig_T2;
int16_t dig_T3;
uint16_t dig_P1;
int16_t dig_P2;
int16_t dig_P3;
int16_t dig_P4;
int16_t dig_P5;
int16_t dig_P6;
int16_t dig_P7;
int16_t dig_P8;
int16_t dig_P9;

int8_t dig_H1;
int16_t dig_H2;
int8_t dig_H3;
int16_t dig_H4;
int16_t dig_H5;
int8_t dig_H6;

void BME280_Write_then_Read(uint8_t *src, uint8_t write_bytes, uint8_t *data_dest, uint8_t read_bytes)
{
    //临时设置I2C从机地址为0x76
    g_i2c_master0.p_api->slaveAddressSet(&g_i2c_master0_ctrl, 0x76, I2C_MASTER_ADDR_MODE_7BIT);
    g_i2c_master0.p_api->write(&g_i2c_master0_ctrl, src, write_bytes, true);
    i2c_wait_tx();
    g_i2c_master0.p_api->read(&g_i2c_master0_ctrl, data_dest, read_bytes, false);
    i2c_wait_rx();
    g_i2c_master0.p_api->slaveAddressSet(&g_i2c_master0_ctrl, 0x3C, I2C_MASTER_ADDR_MODE_7BIT);
}

void BME280_Init(BME_Struct *bme)
{
    uint8_t reg = 0xD0;
    uint8_t write_settings[7] = {0x00};
    uint8_t read_data;
    BME280_Write_then_Read(&reg, 1, &read_data, 1);
    if (read_data != BME280_ID)
    {
        printf("Init BME280 Failed!\n");
        bme->initialized = false;
        return;
    }
    else
    {
        bme->initialized = true;
    }

    write_settings[0] = 0xF2; // 设置湿度采集的寄存器 0xF2
    write_settings[1] = 0x05; // 00000 101 湿度 oversampling x16
    write_settings[2] = 0xF4; // 设置温度采集、气压采集、工作模式的寄存器 0xF4
    write_settings[3] = 0x93; // 100 100 11 温度和气压 oversampling x8，模式为normal
    write_settings[4] = 0xF5; // 配置config寄存器
    write_settings[5] = 0x10; // 000 100 0 0 ，配置滤波器系数为16
    g_i2c_master0.p_api->slaveAddressSet(&g_i2c_master0_ctrl, 0x76, I2C_MASTER_ADDR_MODE_7BIT);
    g_i2c_master0.p_api->write(&g_i2c_master0_ctrl, &write_settings[0], 6, false);
    i2c_wait_tx();
    g_i2c_master0.p_api->slaveAddressSet(&g_i2c_master0_ctrl, 0x3C, I2C_MASTER_ADDR_MODE_7BIT);
    R_BSP_SoftwareDelay(2, BSP_DELAY_UNITS_MILLISECONDS);
    // 校准数据
    BME280_Trimming_Values();
}

void BME280_Trimming_Values()
{
    uint8_t data[33] = {
        0,
    };
    uint8_t reg = 0x88;
    BME280_Write_then_Read(&reg, 1, &data[0], 24);
    R_BSP_SoftwareDelay(5, BSP_DELAY_UNITS_MILLISECONDS); // 适当加延迟否则数据错误
    reg = 0xA1;
    BME280_Write_then_Read(&reg, 1, &data[24], 1);
    R_BSP_SoftwareDelay(5, BSP_DELAY_UNITS_MILLISECONDS); // 适当加延迟否则数据错误
    reg = 0xE1;
    BME280_Write_then_Read(&reg, 1, &data[25], 7);
    R_BSP_SoftwareDelay(5, BSP_DELAY_UNITS_MILLISECONDS); // 适当加延迟否则数据错误

    dig_T1 = (data[1] << 8) | data[0];
    dig_T2 = (data[3] << 8) | data[2];
    dig_T3 = (data[5] << 8) | data[4];

    dig_P1 = (data[7] << 8) | data[6];
    dig_P2 = (data[9] << 8) | data[8];
    dig_P3 = (data[11] << 8) | data[10];
    dig_P4 = (data[13] << 8) | data[12];
    dig_P5 = (data[15] << 8) | data[14];
    dig_P6 = (data[17] << 8) | data[16];
    dig_P7 = (data[19] << 8) | data[18];
    dig_P8 = (data[21] << 8) | data[20];
    dig_P9 = (data[23] << 8) | data[22];

    dig_H1 = data[24];
    dig_H2 = (data[26] << 8) | data[25];
    dig_H3 = data[27];
    dig_H4 = (data[28] << 4) | (data[29] & 0x0F);
    dig_H5 = (data[30] << 4) | ((data[29] >> 4));
    dig_H6 = data[31];
}

// Returns temperature in DegC, double precision. Output value of “51.23” equals 51.23 DegC.
// t_fine carries fine temperature as global value
volatile long signed int t_fine;
double BME280_compensate_T_double(long signed int adc_T)
{
    double var1, var2, T;
    var1 = (((double)adc_T) / 16384.0 - ((double)dig_T1) / 1024.0) * ((double)dig_T2);
    var2 = ((((double)adc_T) / 131072.0 - ((double)dig_T1) / 8192.0) *
            (((double)adc_T) / 131072.0 - ((double)dig_T1) / 8192.0)) *
           ((double)dig_T3);
    t_fine = (long signed int)(var1 + var2);
    T = (var1 + var2) / 5120.0;
    return T;
}
// Returns pressure in Pa as double. Output value of “96386.2” equals 96386.2 Pa = 963.862 hPa
double BME280_compensate_P_double(long signed int adc_P)
{
    double var1, var2, p;
    var1 = ((double)t_fine / 2.0) - 64000.0;
    var2 = var1 * var1 * ((double)dig_P6) / 32768.0;
    var2 = var2 + var1 * ((double)dig_P5) * 2.0;
    var2 = (var2 / 4.0) + (((double)dig_P4) * 65536.0);
    var1 = (((double)dig_P3) * var1 * var1 / 524288.0 + ((double)dig_P2) * var1) / 524288.0;
    var1 = (1.0 + var1 / 32768.0) * ((double)dig_P1);
    if (var1 == 0.0)
    {
        return 0; // avoid exception caused by division by zero
    }
    p = 1048576.0 - (double)adc_P;
    p = (p - (var2 / 4096.0)) * 6250.0 / var1;
    var1 = ((double)dig_P9) * p * p / 2147483648.0;
    var2 = p * ((double)dig_P8) / 32768.0;
    p = p + (var1 + var2 + ((double)dig_P7)) / 16.0;
    return p;
}
// Returns humidity in %rH as as double. Output value of “46.332” represents 46.332 % rH
double bme280_compensate_H_double(long signed int adc_H)
{
    double var_H;
    var_H = (((double)t_fine) - 76800.0);
    var_H = (adc_H - (((double)dig_H4) * 64.0 + ((double)dig_H5) / 16384.0 *
                                                    var_H)) *
            (((double)dig_H2) / 65536.0 * (1.0 + ((double)dig_H6) / 67108864.0 * var_H * (1.0 + ((double)dig_H3) / 67108864.0 * var_H)));
    var_H = var_H * (1.0 - ((double)dig_H1) * var_H / 524288.0);
    if (var_H > 100.0)
        var_H = 100.0;
    else if (var_H < 0.0)
        var_H = 0.0;
    return var_H;
}

void BME280_Get_Data(BME_Struct *bme)
{
    uint8_t dat[8] = {0};
    uint32_t press_t, temp_t, hum_t = 0;
    uint8_t reg = 0xF7;

    BME280_Write_then_Read(&reg, 1, &dat[0], 8);
    R_BSP_SoftwareDelay(2, BSP_DELAY_UNITS_MILLISECONDS);
    press_t = ((((uint32_t)dat[0] << 12) | ((uint32_t)dat[1] << 4)) | ((uint32_t)dat[2] >> 4));
    temp_t = ((((uint32_t)dat[3] << 12) | ((uint32_t)dat[4] << 4)) | ((uint32_t)dat[5] >> 4));
    hum_t = (((uint32_t)dat[6] << 8) | (uint32_t)dat[7]);

    bme->temp = BME280_compensate_T_double(temp_t);
    bme->press = BME280_compensate_P_double(press_t) / 100.0;
    bme->humi = bme280_compensate_H_double(hum_t);
//    printf("temp: %.2lf, humid: %.2lf, pressure: %.2lf\n", bme->temp, bme->humi, bme->press);
}
